package com.usky.dxtop.service.api;


import com.alibaba.fastjson.JSONObject;
import com.usky.dxtop.common.exception.CustomException;
import com.usky.dxtop.common.utils.StringUtils;
import com.usky.dxtop.common.utils.http.HttpUtils;
import com.usky.dxtop.common.utils.sign.Md5Utils;
import org.apache.poi.hssf.record.SSTRecord;
import org.apache.poi.ss.usermodel.FontUnderline;
import sun.security.timestamp.TSRequest;

import java.util.*;
import java.util.function.Function;

/**
 * @author yq
 * @date 2021/8/26 9:06
 */
public class OneCardApi {

    private static final String URL = "http://10.208.2.158:7002/";

    private static final Integer VERSION = 3;
    /**
     * 卡消费
     */
    public static final String CARD_PAY_URL = String.format("%s%s",URL,"partner/card/loss");
    /**
     * 消费检查
     */
    public static final String CARD_PAYCHECK = String.format("%s%s",URL,"caih/reissue");
    /**
     * 餐厅编号
     */
    private static final String SHOP = "101";
    /**
     * 终端编号
     */
    private static final String TERM = "108";

    /**
     * 终端操作员
     */
    private static final String OPER = "admin";


    private static final String KEY = "951b19605fce4132bd40fbfe3f85a1a6";

    /**
     * 公用请求
     * @return
     */
    public static TreeMap<String,String> commonParam(){
        TreeMap<String,String> treeMap = new TreeMap<>();
        treeMap.put("shop",SHOP);
        treeMap.put("term",TERM);
        treeMap.put("oper",OPER);
        treeMap.put("d",new Date().toString());
        return treeMap;
    }

    /**
     * 卡挂失
     * @param cid
     * @param cno
     * @param msg
     * @return
     */
    public static TreeMap<String,String> cardLoss(String cid,String cno,String msg){
        TreeMap<String, String> treeMap = commonParam();
        treeMap.put("cid",cid);
        treeMap.put("cno",cno);
        treeMap.put("msg",msg);
        List<String> paramList = getParamList(treeMap);
        treeMap.put("sig",sign(paramList));
        return treeMap;
    }

    public static TreeMap<String,String> cardReissue(String cid,String cidNew){
        TreeMap<String, String> treeMap = commonParam();
        treeMap.put("cid",cid);
        treeMap.put("cid_new",cidNew);
        treeMap.put("v", String.valueOf(VERSION));
        treeMap.put("format", "json");
        List<String> paramList = getParamList(treeMap);
        treeMap.put("sig",sign(paramList));
        return treeMap;
    }


    /**
     * 卡消费
     * @return
     */
    public static TreeMap<String,String> cardPay(){
        return null;
    }

    public static TreeMap<String,String> cardPayBack(Integer seq){
        return null;
    }



    /**
     * 验签
     * @param list
     * @return
     */
    public static String sign(List<String> list){
        String[] strs = new String[list.size()];
        list.toArray(strs);
        Arrays.sort(strs , String.CASE_INSENSITIVE_ORDER);
        String str = "";
        for(int i=0; i < strs.length; i++) str += strs[i];
        str += KEY;
        return Md5Utils.hash(str);
    }


    /**
     * 获取list请求参数
     * @param params
     * @return
     */
    public static List<String> getParamList(TreeMap<String, String> params){
        List<String> list = new ArrayList<>();
        for (Map.Entry<String, String> entry : params.entrySet()) {
            StringBuilder buf = new StringBuilder();
            if (!StringUtils.isBlank(entry.getValue()) && !"sign".equals(entry.getKey())) {
                list.add(buf.append(entry.getKey()).append("=").append(entry.getValue()).toString());
            }
        }
        return list;
    }


    /**
     * 拼接请求参数
     * @param params
     * @return
     */
    public static String joinParam(TreeMap<String, String> params){
        StringBuilder buf = new StringBuilder();
        for (Map.Entry<String, String> entry : params.entrySet()) {
            String value = "";
            if (StringUtils.isNotBlank(entry.getValue())){
                value = entry.getValue();
            }
            if (!"sign".equals(entry.getKey())) {
                buf.append(entry.getKey()).append("=").append(value).append("&");
            }
        }
        String changeBuf = buf.toString();
        return changeBuf.substring(0, changeBuf.length() - 1);
    }

    public Boolean commonRequest(String url, String param, Function<JSONObject,Boolean> function){
        String result = HttpUtils.sendPost(url, JSONObject.toJSONString(param), null);
        JSONObject jsonObject = JSONObject.parseObject(result);
        if ("1".equals(jsonObject.get("code").toString())) {
            return function.apply(jsonObject);
        } else {
            throw new CustomException("接口异常"+jsonObject.get("msg"));
        }
    }
}
