package com.usky.dxtop.service.job;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.dxtop.common.utils.StringUtils;
import com.usky.dxtop.common.utils.http.HttpUtils;
import com.usky.dxtop.model.Dept;
import com.usky.dxtop.model.Staff;
import com.usky.dxtop.model.SysDept;
import com.usky.dxtop.service.DeptService;
import com.usky.dxtop.service.StaffService;
import com.usky.dxtop.service.api.SmApi;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;

/**
 * 世贸定时任务
 *
 * @author yq
 * @date 2021/9/28 14:47
 */
@Component("smJob")
@Slf4j
public class SmJob {

    @Autowired
    private StaffService staffService;

    @Autowired
    private DeptService deptService;

//    @Scheduled(cron = "0 0 0 */1 * ?")
    public void personExecute(String param) {
        PersonParam personParam = JSON.parseObject(param, PersonParam.class);
        personApi(personParam);
        log.info("smJob-person 定时任务完成");
    }

    public void groupExecute(String param) {
        GroupParam groupParam = JSON.parseObject(param, GroupParam.class);
        groupApi(groupParam);
        log.info("smJob-group 定时任务完成");
    }


    public void  personApi(PersonParam personParam){
        TreeMap<String, Object> treeMap;
        if (null != personParam){
            treeMap = SmApi.generatePersonParam(personParam.getId(),
                    personParam.getDeptIds(), personParam.getCreateTime(), personParam.getBlurry());
        }else {
            treeMap = SmApi.generatePersonParam(null,
                    null, null, null);
        }
        List<Dept> depts = deptService.list();
        List<Staff> checkStaffList = new ArrayList<>();
        common(SmApi.PERSON_URL,treeMap,staffList -> {
            List<Staff> list = staffList.toJavaList(Staff.class);
            list.forEach(date -> {
                try {
                    if (enhanceStaff(date,depts)){
                        checkStaffList.add(date);
                        //发送人员信息
                        if (StringUtils.isNotBlank(date.getCardId()) && Double.parseDouble(date.getCardId()) > 60000){
                            staffService.personSendMessage(date,date.getAddOrUp());
                            //发送人脸信息
                            if (StringUtils.isNotBlank(date.getFaceId())){
                                staffService.faceSendMessage(date);
                            }
                        }
                    }
                }catch (Exception e){
                    log.error("smJob-person:"+e.getMessage());
                }
            });
            staffService.saveOrUpdateBatch(checkStaffList);
            checkStaffList.clear();
            return list.isEmpty();
        });
    }

    /**
     * 增强人员信息
     * @param data
     */
    public boolean enhanceStaff(Staff data,List<Dept> depts){
        data.setCardId(new BigDecimal(data.getCardId()).toString());
        LambdaQueryWrapper<Staff> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Staff::getId,data.getId());
        Staff staff = staffService.getOne(queryWrapper);
        Map deptMap = JSON.parseObject(data.getDept(), Map.class);
        return depts.stream().filter(dept -> dept.getId().equals(deptMap.get("id").toString())).findAny()
                .map(s -> {
                    data.setDeptId(s.getDId());
                    if (null == staff){
                        data.setAddOrUp(1);
                    }else {
                        data.setAddOrUp(2);
                        data.setSId(staff.getSId());
                    }
                    return true;
                }).orElse(false);

    }

    public void groupApi(GroupParam groupParam){
        TreeMap<String, Object> treeMap;
        if (null != groupParam){
            treeMap = SmApi.generateDeptParam(groupParam.getCreateTime(),
                    groupParam.getName(),groupParam.getPid(),groupParam.getEnable());
        }else {
            treeMap = SmApi.generateDeptParam(null,
                    null,null,null);
        }
        List<Dept> checkDeptList = new ArrayList<>();
        common(SmApi.DEPT_URL,treeMap,deptList -> {
            List<Dept> list = deptList.toJavaList(Dept.class);
            list.forEach(date -> {
                try {
                    //只负责同步
                    LambdaQueryWrapper<Dept> queryWrapper = Wrappers.lambdaQuery();
                    queryWrapper.eq(Dept::getId,date.getId());
                    if (StringUtils.isBlank(date.getPid())){
                        date.setPid("0");
                    }
                    Dept dept = deptService.getOne(queryWrapper);
                    if (null != dept){
                        date.setDId(dept.getDId());
                    }
                    checkDeptList.add(date);
                }catch (Exception e){
                    log.error("smJob-group:"+e.getMessage());
                }
            });
            deptService.saveOrUpdateBatch(checkDeptList);
            checkDeptList.clear();
            return list.isEmpty();
        });
        //拼接父子结构发送数据
        List<Dept> list = deptService.list();
        deptService.buildDeptTree(list);
    }

    public void common(String url, TreeMap<String,Object> param, Function<JSONArray,Boolean> function){
        boolean isNext = true;
        int page = 0;
        int size = 200;
        while (isNext) {
            try {
                String personUrl = String.format("%s?page=%s&size=%s", url, page, size);
                String result = HttpUtils.sendPost(personUrl, JSONObject.toJSONString(param), SmApi.sign(param));
                JSONObject jsonObject = JSONObject.parseObject(result);
                if ("200".equals(jsonObject.get("status").toString())) {
                    JSONObject data = jsonObject.getJSONObject("data");
                    JSONArray content = data.getJSONArray("content");
                    if (content != null && content.size() > 0) {
                        if (function.apply(content)) {
                            isNext = false;
                        } else {
                            page++;
                        }
                    } else {
                        isNext = false;
                    }
                } else {
                    break;
                }
            } catch (Exception e) {
                log.info("smJob异常" + e);
                break;
            }
        }
    }



    @Data
    public static class PersonParam {
        private String id;
        private List<String> deptIds;
        private List<Date> createTime;
        private String blurry;
    }


    @Data
    public static class GroupParam {
        private String name;
        private Boolean enable;
        private List<Date> createTime;
        private String pid;
    }
}
